@@ -158,11 +158,14 @@ def buildquery(json_object):
 
     if 'field' in json_object.keys():
         # In this case the json_object is an object which describes a single query
-        if json_object.get("operator") == 'IN':
+        if json_object.get("operator").upper() == 'IN':
             value = json_object.get("value")
             value = tuple(num for num in value.replace('(', '').replace(')', '').replace('...', '').split(', '))
             return json_object.get("field") + " " + json_object.get("operator") + " " + str(value) + " "
-        if json_object.get("operator") != 'BETWEEN':
+        if json_object.get("operator").upper() == 'LIKE':
+            return json_object.get("field") + " " + json_object.get("operator") + " '%" + json_object.get(
+                "value") + "%' "
+        if json_object.get("operator").upper() != 'BETWEEN':
             return json_object.get("field") + " " + json_object.get("operator") + " '" + json_object.get("value") + "' "
         else:
             return json_object.get("field") + " " + json_object.get("operator") + " '" + json_object.get(
@@ -297,6 +300,10 @@ def run_external_workflow(conf, task_id, vtiger_access):
             search_object = external_workflow_config.get('search_object', '')
             print(f"Data: {data} \n Workflow: {workflow} \n Event Type: {event_type} \n Search Object: {search_object}")
             if search_object and search_object.get('condition_object', ''):
+                if 'env' in data.keys():
+                    env_variables = data.get('env')
+                else:
+                    env_variables = {}
                 rule = search_object.get('rule', '')
 
                 # prepare query
@@ -376,12 +383,16 @@ def run_external_workflow(conf, task_id, vtiger_access):
 
                         if event_type == 'import' or event_type == 'manual':
                             # if event type = import/ manual then pass list
+                            response['result']['env'] = env_variables
 
                             trigger_workflow(workflow, event_type, response.get('result'),
-                                             vtiger_access.get('service_url'), vtiger_access.get('api_access_key'), module)
+                                             vtiger_access.get('service_url'), vtiger_access.get('api_access_key'),
+                                             module)
                         else:
+                            response['result'][0]['env'] = env_variables
                             trigger_workflow(workflow, event_type, response.get('result')[0],
-                                             vtiger_access.get('service_url'), vtiger_access.get('api_access_key'), module)
+                                             vtiger_access.get('service_url'), vtiger_access.get('api_access_key'),
+                                             module)
             elif workflow and event_type and data and source_module:
                 # trigger without condition
                 if event_type == 'import' or event_type == 'manual':
@@ -589,6 +600,10 @@ def invoke_set_value_task(conf, task_id, vtiger_access, record_id=None):
                 if response:
                     print(response)
                     if response.get('result') and set_value_fields:
+                        if type(response.get('result')) == list:
+                            record_id = response.get('result')[0]['id']
+                        else:
+                            record_id = response.get('result').get('id')
                         set_value_response = trigger_set_value_task(set_value_fields, conf, rule, session_name,
                                                                     vtiger_access, record_id)
                         return set_value_response
@@ -900,3 +915,163 @@ def get_record_details_from_crm(vtiger_access, data):
 
     else:
         print("unable to get id & data from conf object")
+
+
+def get_conf(workflow, vtiger_access):
+    endpoint = "{service_url}/api/v1/get-workflow-configs/{workflow}".format(
+        service_url=vtiger_access.get('service_url'), workflow=workflow)
+
+    headers = {'content_type': 'application/json', 'Authorization': vtiger_access.get('api_access_key')}
+    response, status = invoke_http_request(endpoint, 'GET', headers, payload=None, json_data=None, timeout=61)
+
+    if is_success_request(status) and response:
+        print("response from get conf API: ", response)
+        return response
+    else:
+        print("unable to get response from get conf API: ", response, status)
+
+def trigger_workflow_with_fetch_configs(context, workflow, data, vtiger_access):
+    """ call API to trigger any workflow
+        Required params: 1. context 2. workflow 3. data 4. vtiger_access"""
+    from airflow.operators.trigger_dagrun import TriggerDagRunOperator
+    conf = get_conf(workflow, vtiger_access)
+    conf["data"] = data
+
+    TriggerDagRunOperator(
+        task_id="trigger_dagrun",
+        trigger_dag_id=workflow,
+        conf=conf,
+    ).execute(context)
+
+
+def run_external_workflow_with_fetch_configs(context, task_id, vtiger_access):
+    """ this function will get check if conditions are satisfied for triggering external workflow or not.
+        input:  1.context : context object
+                2.task_id: current task id
+                3.vtiger_access: vtiger_access config dict
+
+        1. get external_workflow_config : config dictionary containing workflow, event_type, search_object
+        2. get configs and create query.
+        3. replace data using json logic.
+        4. call trigger workflow function if conditions are satisfied.
+        """
+
+    conf = context["dag_run"].conf
+    if conf.get('external_workflow_config', ''):
+
+        external_workflow_config = {}
+        if task_id in conf.get('external_workflow_config', ''):
+            external_workflow_config = conf.get('external_workflow_config').get(task_id, '')
+
+        if external_workflow_config:
+
+            data = conf.get('data', '')
+            workflow = external_workflow_config.get('workflow', '')
+            event_type = external_workflow_config.get('event_type', '')
+
+            search_object = external_workflow_config.get('search_object', '')
+            print(f"Data: {data} \n Workflow: {workflow} \n Event Type: {event_type} \n Search Object: {search_object}")
+            if search_object and search_object.get('condition_object', ''):
+                if 'env' in data.keys():
+                    env_variables = data.get('env')
+                else:
+                    env_variables = {}
+                rule = search_object.get('rule', '')
+
+                # prepare query
+                condition = buildquery(search_object.get('condition_object'))
+                module = search_object.get('search_module').get('name')
+                limit = str(search_object.get('fetch_record', ''))
+
+                if event_type != 'import' and event_type != 'manual':
+                    limit = '1'
+
+                order_by = ''
+                if search_object.get('sort', ''):
+                    order_by = search_object.get('sort').get('column') + " " + search_object.get('sort').get(
+                        'type', "")
+
+                if limit and order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                        limit=limit)
+                elif limit:
+                    query = 'SELECT * FROM {module} WHERE {condition} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        limit=limit)
+                elif order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                    )
+                else:
+                    query = 'SELECT * FROM {module} WHERE {condition};'.format(
+                        module=module,
+                        condition=condition,
+                    )
+
+                # replace variable name with data using JSON_LOGIC.
+                print(f"Query: {query}")
+                if rule:
+                    query = json_logic_replace_data(rule, conf, query)
+
+                session_name = vtiger_access.get('vtiger_session', '')
+                print("Session name: {}".format(session_name))
+                if not session_name:
+                    print("unable to get session id from vtiger access. Please try again")
+                    return None
+
+                url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
+                    sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
+                print(f"Url: {url}")
+                headers = {'content-type': 'application/json'}
+                request_type = 'GET'
+                response, status = invoke_http_request(url, request_type, headers)
+
+                print(f"Response: {response}")
+
+                if response:
+                    if not response.get('success', ''):
+                        if response.get('error', ''):
+                            if response.get('error').get('code') in ["INVALID_USER_CREDENTIALS", "INVALID_SESSIONID"]:
+                                session_name = get_session_name(vtiger_access)
+
+                                # retry to execute query
+
+                                url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
+                                    sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
+                                response, status = invoke_http_request(url, request_type, headers)
+
+                                vtiger_access['vtiger_session'] = session_name
+                                Variable.update(key='vtiger_access', value=vtiger_access, serialize_json=True)
+
+                if response:
+                    # trigger only if condition is satisfied
+                    if response.get('result') and workflow and event_type and module:
+
+                        if event_type == 'import' or event_type == 'manual':
+                            # if event type = import/ manual then pass list
+                            response['result']['env'] = env_variables
+
+                            trigger_workflow_with_fetch_configs(context, workflow, response.get('result'),vtiger_access)
+                        else:
+                            response['result'][0]['env'] = env_variables
+                            trigger_workflow_with_fetch_configs(context, workflow, response.get('result')[0],vtiger_access)
+            elif workflow and event_type and data:
+                # trigger without condition
+                if event_type == 'import' or event_type == 'manual':
+                    list_of_data = [data]
+                    trigger_workflow_with_fetch_configs(context, workflow, list_of_data, vtiger_access)
+                else:
+                    trigger_workflow_with_fetch_configs(context, workflow, data, vtiger_access)
+
+            else:
+                print("Unable to get required data: workflow, event_type, data and source_module from payload ")
+        else:
+            print("external_workflow_config not found for task id:", task_id)
+    else:
+        print("external_workflow_config not found in conf")
